@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+import json
 import logging
 import operator
 import re
@@ -6,7 +9,9 @@ import struct
 import sys
 import time
 from datetime import datetime
+from typing import Any, Optional
 
+import attrs
 import numpy as np
 import serial.tools.list_ports
 from packaging import version
@@ -22,6 +27,43 @@ try:
 except ImportError:
     pg = None
 
+try:
+    from acconeer.exptool._winusbcdc.winusbpy import WinUsbPy
+except ImportError:
+    WinUsbPy = None
+
+
+@attrs.frozen(kw_only=True)
+class USBDevice:
+    vid: int
+    pid: int
+    serial: Optional[str] = None
+    name: str
+
+    def to_dict(self) -> dict[str, Any]:
+        return attrs.asdict(self)
+
+    @classmethod
+    def from_dict(cls, d: dict) -> USBDevice:
+        return cls(**d)
+
+    def to_json(self) -> str:
+        return json.dumps(self.to_dict())
+
+    @classmethod
+    def from_json(cls, json_str: str) -> USBDevice:
+        return cls.from_dict(json.loads(json_str))
+
+    def __str__(self) -> str:
+        return self.name
+
+
+_USB_IDS = [  # (vid, pid, 'model number')
+    (0x0483, 0xA41D, "XC120"),
+    (0x0483, 0xA42C, "XC120"),
+    (0x0483, 0xA42D, "XC120"),
+]
+
 
 class ExampleInterruptHandler:
     def __init__(self):
@@ -80,6 +122,43 @@ def set_loglevel(level):
     log.setLevel(level)
 
 
+def tag_serial_ports_objects(port_infos):
+    PRODUCT_REGEX = r"[X][A-Z]\d{3}"
+
+    port_tag_tuples = []
+
+    for _, port_object in enumerate(port_infos):
+        desc = port_object.product or port_object.description
+
+        match = re.search(PRODUCT_REGEX, desc)
+        if match is None:
+            for vid, pid, model_number in _USB_IDS:
+                if port_object.vid == vid and port_object.pid == pid:
+                    port_tag_tuples.append((port_object, model_number))
+                    break
+            else:
+                port_tag_tuples.append((port_object, None))
+        elif "xe132" in match.group().lower():
+            if version.parse(serial.__version__) >= version.parse("3.5"):
+                # Special handling of xe132 with pyserial >= 3.5
+                interface = port_object.interface
+
+                if interface and "enhanced" in interface.lower():
+                    # Add the "enhanced" interface
+                    port_tag_tuples.append((port_object, "XE132"))
+                else:
+                    # Add the "standard" interface but don't tag it
+                    port_tag_tuples.append((port_object, None))
+
+            else:  # pyserial <= 3.4
+                # Add "?" to both to indicate that it could be either.
+                port_tag_tuples.append((port_object, "XE132 (?)"))
+        else:
+            port_tag_tuples.append((port_object, match.group()))
+
+    return port_tag_tuples
+
+
 def tag_serial_ports(port_infos):
     """
     Returns every port and Acconeer model number in a tuple if it's an
@@ -94,43 +173,8 @@ def tag_serial_ports(port_infos):
         `serial.tools.list_ports.comports()`
     :returns: List of tuples [(<port>, <model number> or None), ...]
     """
-    PRODUCT_REGEX = r"[X][A-Z]\d{3}"
-    USB_IDS = [  # (vid, pid, 'model number')
-        (0x0483, 0xA42D, "XC120"),
-    ]
-
-    port_tag_tuples = []
-
-    for _, port_object in enumerate(port_infos):
-        port = port_object.device
-        desc = port_object.product or port_object.description
-
-        match = re.search(PRODUCT_REGEX, desc)
-        if match is None:
-            for vid, pid, model_number in USB_IDS:
-                if port_object.vid == vid and port_object.pid == pid:
-                    port_tag_tuples.append((port, model_number))
-                    break
-            else:
-                port_tag_tuples.append((port, None))
-        elif "xe132" in match.group().lower():
-            if version.parse(serial.__version__) >= version.parse("3.5"):
-                # Special handling of xe132 with pyserial >= 3.5
-                interface = port_object.interface
-
-                if interface and "enhanced" in interface.lower():
-                    # Add the "enhanced" interface
-                    port_tag_tuples.append((port, "XE132"))
-                else:
-                    # Add the "standard" interface but don't tag it
-                    port_tag_tuples.append((port, None))
-
-            else:  # pyserial <= 3.4
-                # Add "?" to both to indicate that it could be either.
-                port_tag_tuples.append((port, "XE132 (?)"))
-        else:
-            port_tag_tuples.append((port, match.group()))
-
+    port_tag_tuples = tag_serial_ports_objects(port_infos)
+    port_tag_tuples = [(port.device, tag) for (port, tag) in port_tag_tuples]
     return port_tag_tuples
 
 
@@ -181,6 +225,32 @@ def autodetect_serial_port():
     sys.exit()
 
 
+def get_usb_devices():
+    usb_devices = []
+
+    if WinUsbPy is None:
+        raise ImportError("USB ports only supported for Windows")
+
+    winusbpy = WinUsbPy()
+    all_usb_devices = winusbpy.list_usb_devices(deviceinterface=True, present=True)
+
+    for name, path in all_usb_devices.items():
+        found = False
+        pattern = r"(.*)#vid_(?P<vid>\w+)&pid_(?P<pid>\w+)(.*)"
+        match = re.fullmatch(pattern, path)
+        groups = match.groupdict()
+        v = int(f"0x{groups['vid']}", 16)
+        p = int(f"0x{groups['pid']}", 16)
+        for vid, pid, _ in _USB_IDS:
+            if v == vid and p == pid:
+                found = True
+
+        if found:
+            usb_devices.append(USBDevice(vid=v, pid=p, serial=None, name=name))
+
+    return usb_devices
+
+
 def color_cycler(i=0):
     category10 = [
         "#1f77b4",
